package com.muyu.config.interceptor;

import com.github.benmanes.caffeine.cache.Cache;
import com.muyu.system.constants.TokenConstants;
import com.muyu.system.context.SystemUserContext;
import com.muyu.system.domain.LoginUserInfo;
import com.muyu.system.exception.BasicException;
import com.muyu.utils.JwtUtils;
import io.jsonwebtoken.Claims;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

/**
 * 用户登录拦截器
 */
@Component
public class LoginInterceptor implements HandlerInterceptor {

    @Resource( name = "loginUserCache")
    private Cache<String, LoginUserInfo> loginUserInfoCache;

    /**
     * 在请求被处理之前进行操作
     *
     * @param request
     * @param response
     * @param handler
     * @return 返回true表示继续执行后续的拦截器或Controller；返回false则不再执行后续的拦截器或Controller
     * @throws Exception
     */
    @Override
    public boolean preHandle(HttpServletRequest request, @NotNull HttpServletResponse response, @NotNull Object handler) throws Exception {
        // 判断当前请求是否需要被拦截
        String token = request.getHeader(TokenConstants.AUTHENTICATION);
        if (StringUtils.isEmpty(token)){
            throw new BasicException(501,"未授权请求，请重新授权");
        }
        Claims claims = JwtUtils.parseToken(token);
        String userKey = JwtUtils.getUserKey(claims);
        LoginUserInfo loginUserInfo = loginUserInfoCache.getIfPresent(userKey);
        if (loginUserInfo == null){
            throw new BasicException(502,"用户授权过期，请重新登录");
        }
        SystemUserContext.set(loginUserInfo);
        return true;
    }

    /**
     * 在请求被处理之后但视图还未渲染时进行操作
     *
     * @param request
     * @param response
     * @param handler
     * @param modelAndView
     * @throws Exception
     */
    @Override
    public void postHandle(@NotNull HttpServletRequest request, @NotNull HttpServletResponse response, @NotNull Object handler, ModelAndView modelAndView) throws Exception {

    }

    /**
     * 在整个请求结束之后（包括视图已经渲染）进行操作
     *
     * @param request
     * @param response
     * @param handler
     * @param ex
     * @throws Exception
     */
    @Override
    public void afterCompletion(@NotNull HttpServletRequest request, @NotNull HttpServletResponse response, @NotNull Object handler, Exception ex) throws Exception {
        SystemUserContext.remove();
    }
}

