package com.muyu.web.domain.model;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.log4j.Log4j2;

import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Consumer;

/**
 * @author DongZeLiang
 * @version 1.0
 * @description 任务执行模型
 * @date 2023/12/4
 */
@Data
@Log4j2
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class TaskModel {

    /**
     * 任务状态 默认为false状态
     * false为未执行任务
     * true为执行任务
     */
    private final AtomicBoolean unifiedStatus = new AtomicBoolean(Boolean.FALSE);

    /**
     * 堵塞计数器
     */
    private CountDownLatch countDownLatch ;

    /**
     * 任务执行堵塞队列
     */
    private LinkedBlockingQueue<String> vehicleTaskQueue = new LinkedBlockingQueue<>();

    /**
     * 任务是否可以执行
     *  true 为未执行任务-可以执行任务
     *  false为执行任务-不可执行任务
     * @return 是否有任务执行
     */
    public boolean isExecution(){
        return !unifiedStatus.get();
    }

    /**
     * 任务名称
     */
    private String taskName;
    /**
     * 任务执行总数
     */
    private Integer taskExecutionSum = 0;

    /**
     * 任务执行时间
     */
    private long taskStartTime;


    /**
     * 任务成功执行总数
     */
    private AtomicInteger taskSuccessSum = new AtomicInteger();


    /**
     * 任务执行失败总数
     */
    private AtomicInteger taskErrorSum = new AtomicInteger();


    /**
     * 判断是否有下个任务
     * @return true 下一个任务可执行
     */
    public boolean hashNext(){
        return !vehicleTaskQueue.isEmpty();
    }

    /**
     * 获取下一个任务节点
     * @return 任务VIN
     */
    public String next(){
        return vehicleTaskQueue.poll();
    }

    /**
     * 提交当前线程任务
     * @param taskName 任务名称
     * @param vehicleVinList 提交车辆VIN集合
     */
    public void submit(String taskName, List<String> vehicleVinList, Consumer<String> consumer){
        if (!this.isExecution()){
            throw new RuntimeException("["+this.taskName+"]的任务正在进行中，请等待任务执行完成再次发布一键任务");
        }
        if (vehicleVinList.isEmpty()){
            throw new RuntimeException("无需执行");
        }
        unifiedStatus.set(Boolean.TRUE);
        this.taskName = taskName;
        this.countDownLatch = new CountDownLatch(vehicleVinList.size());
        this.vehicleTaskQueue.addAll(vehicleVinList);
        this.taskExecutionSum = vehicleVinList.size();
        this.taskSuccessSum = new AtomicInteger();
        this.taskErrorSum = new AtomicInteger();
        this.taskStartTime = System.currentTimeMillis();
        new Thread(() -> {
            try {
                while (hashNext()){
//                    Thread.sleep(500);
                    String vin = next();
                    try {
                        consumer.accept(vin);
                        incrementSuccess();
                    }catch (Exception e){
                        log.error("任务:[{}] vin:[{}] 异常：[{}]",taskName,vin,e.getMessage(),e);
                        incrementError();
                    }
                }
            }catch (Exception e){
                log.error("任务:[{}] 异常：[{}]",taskName,e.getMessage(),e);
            }finally {
                down();
            }
        }).start();
        log.info("[{}]任务执行开始", this.taskName);
    }

    /**
     * 执行结束
     */
    public void down(){
        log.info("[{}]任务执行结束，耗时：[{}]MS", this.taskName, System.currentTimeMillis() - taskStartTime);
        this.taskName = null;
        this.taskExecutionSum = 0;
        this.taskSuccessSum.set(0);
        this.taskErrorSum.set(0);
        unifiedStatus.set(Boolean.FALSE);
    }

    /**
     * 累计成功
     */
    public void incrementSuccess(){
        this.taskSuccessSum.incrementAndGet();
    }
    /**
     * 获取成功数量
     */
    public int getSuccessSum(){
       return this.taskSuccessSum.get();
    }

    /**
     * 累计失败
     */
    public void incrementError(){
        this.taskErrorSum.incrementAndGet();
    }
    /**
     * 获取失败数量
     */
    public int getErrorSum(){
        return this.taskErrorSum.get();
    }
}
